/**
 * @module libflitter/config/ConfigUnit
 */

const CanonicalUnit = require('../canon/CanonicalUnit')
const helpers = require('../utility/Helpers')
const path = require('path')

/**
 * Unit to load and manage config files.
 * @extends module:libflitter/canon/CanonicalUnit~CanonicalUnit
 */
class ConfigUnit extends CanonicalUnit {
    /**
     * Name of the service provided by this unit: 'configs'
     * @returns {string} - 'configs'
     */
    static get name() {
        return 'configs'
    }

    /**
     * Instantiate the unit.
     * @param {string} [base_directory = './config'] - the base directory
     */
    constructor(base_directory = './config') {
        super(base_directory)

        /**
         * The canonical name of the item.
         * @type {string} = 'config'
         */
        this.canonical_item = 'config'

        /**
         * The file extension of the canonical item files.
         * @type {string} - '.config.js'
         */
        this.suffix = '.config.js'
    }

    /**
     * Initialize the unit. Load the dotenv configuration.
     * @param app
     * @returns {Promise<void>}
     */
    async go(app) {
        require('dotenv').config()
        global.env = helpers.env.bind(helpers)
        await super.go(app)
        delete global.env
    }

    /**
     * A helper function that calls {@link module:libflitter/config/ConfigUnit~ConfigUnit#get}, but
     * guarantees an object is returned, even if no config is found for the given accessor.
     * @param {string} accessor - period-delineated access string
     * @param {Object} [merge = {}] - default values to override
     * @return {Object}
     */
    guarantee(accessor, merge = {}) {
        const config = this.get(accessor)
        const grant = config ? config : {}
        return {...merge, ...grant}
    }

    /**
     * Get the templates provided by this unit.
     * Currently, "config" provided by {@link module:libflitter/templates/config}
     * @returns {{config: {template: (config|(function(string): string)|*), extension: string, directory: string}}}
     */
    templates() {
        return {
            config: {
                template: require('../templates/config'),
                directory: this.directory,
                extension: '.config.js'
            }
        }
    }

    /**
     * Get the fully-qualified path to the migrations provided by this unit.
     * @returns {string}
     */
    migrations(){
        return path.resolve(__dirname, 'migrations')
    }
}

module.exports = exports = ConfigUnit
