/**
 * @module libflitter/utility/services/Output
 */

const Service = require('../../NamedService')
const color = require('colors/safe')

/**
 * Service for managing output to the console based on logging level.
 * @extends module:flitter-di/src/Service~Service
 */
class Output extends Service {
    /**
     * Get the name of this service: 'output'
     * @returns {string} - 'output'
     */
    static get name() { return 'output' }

    constructor() {
        super()
        /**
         * The logging level.
         * @type {number}
         */
        this.level = 1

        /**
         * If true, includes a timestamp in the output. Default false.
         * @type {boolean}
         */
        this.timestamp = false
    }

    /**
     * Get a color-formatted timestamp string.
     * @returns {string}
     * @private
     */
    __timestamp() {
        return this.timestamp ? color.cyan(`[${(new Date).toLocaleString()}]`) : ''
    }

    /**
     * Send a [DEBUG] message.
     * The logging level must be greater than or equal to the message level for it to display.
     * @param {string} msg
     * @param {number} [level = 5] - the message level
     */
    debug(msg, level = 5) {
        if ( this.level < level ) return;
        console.log(color.magenta('   [DEBUG]'), this.__timestamp(), msg)
    }

    /**
     * Send an [INFO] message.
     * The logging level must be greater than or equal to the message level for it to display.
     * @param {string} msg
     * @param {number} [level = 4] - the message level
     */
    info(msg, level = 4) {
        if ( this.level < level ) return;
        console.log(color.blue('    [INFO]'), this.__timestamp(), msg)
    }

    /**
     * Send a [MESSAGE] message.
     * The logging level must be greater than or equal to the message level for it to display.
     * @param {string} msg
     * @param {number} [level = 3] - the message level
     */
    message(msg, level = 3) {
        if ( this.level < level ) return;
        console.log(color.cyan(' [MESSAGE]'), this.__timestamp(), msg)
    }

    /**
     * Send a [WARNING] message.
     * The logging level must be greater than or equal to the message level for it to display.
     * @param {string} msg
     * @param {number} [level = 2] - the message level
     */
    warn(msg, level = 2) {
        if ( this.level < level ) return;
        console.log(color.yellow(' [WARNING]'), this.__timestamp(), msg)
    }

    /**
     * Send an [ERROR] message.
     * The logging level must be greater than or equal to the message level for it to display.
     * @param {string} msg
     * @param {number} [level = 1] - the message level
     */
    error(msg, level = 1) {
        if ( this.level < level ) return;
        console.log(color.red('   [ERROR]'), this.__timestamp(), msg)
    }

    /**
     * Send a [SUCCESS] message.
     * The logging level must be greater than or equal to the message level for it to display.
     * @param {string} msg
     * @param {number} [level = 1] - the message level
     */
    success(msg, level = 1) {
        if ( this.level < level ) return;
        console.log(color.green(' [SUCCESS]'), this.__timestamp(), msg)
    }
}

module.exports = exports = Output
